@@ -106,20 +106,24 @@
 (export gnc:dump-lot)
 
 (define (list-ref-safe list elt)
-  (and (> (length list) elt)
-       (list-ref list elt)))
+  (and (pair? list)
+       (if (<= elt 0)
+           (car list)
+           (list-ref-safe (cdr list) (1- elt)))))
 
 (define (list-set-safe! l elt val)
-  (unless (list? l)
-    (set! l '()))
-  (if (> (length l) elt)
-      (list-set! l elt val)
-      (let loop ((filler (list val))
-                 (i (length l)))
-        (if (< i elt)
-            (loop (cons #f filler) (1+ i))
-            (set! l (append! l filler)))))
-  l)
+  (define (extend-tail i)
+    (if (>= i elt)
+        (list val)
+        (cons #f (extend-tail (1+ i)))))
+  (if (null? l)
+      (extend-tail 0)
+      (let loop ((i 0) (curr l))
+        (cond
+         ((not (pair? curr)) (error "list-set-safe: improper list"))
+         ((>= i elt)         (set-car! curr val) l)
+         ((null? (cdr curr)) (set-cdr! curr (extend-tail (1+ i))) l)
+         (else               (loop (1+ i) (cdr curr)))))))
 
 ;; Just for convenience. But in reports you should rather stick to the
 ;; style-info mechanism and simple plug the <gnc-monetary> into the
@@ -217,14 +221,11 @@
 ;; Get the list of all different commodities that are used within the
 ;; 'accounts', excluding the 'exclude-commodity'.
 (define (gnc:accounts-get-commodities accounts exclude-commodity)
-  (delete exclude-commodity
-          (sort-and-delete-duplicates
-           (map xaccAccountGetCommodity accounts)
-           (lambda (a b)
-             (gnc:string-locale<? (gnc-commodity-get-unique-name a)
-                                  (gnc-commodity-get-unique-name b)))
-           gnc-commodity-equiv)))
-
+  (if (null? accounts)
+      '()
+      (let ((comm (xaccAccountGetCommodity (car accounts)))
+            (accum (gnc:accounts-get-commodities (cdr accounts) exclude-commodity)))
+        (if (or (equal? exclude-commodity comm) (member comm accum)) accum (cons comm accum)))))
 
 ;; Returns the depth of the current account hierarchy, that is, the
 ;; maximum level of subaccounts in the tree
@@ -475,41 +476,9 @@
           (nosplit->elt #f)
           (split->date #f)
           (split->elt xaccSplitGetBalance))
-  (define to-date (or split->date (compose xaccTransGetDate xaccSplitGetParent)))
-  (define (less? a b) (< (to-date a) (to-date b)))
-
-  (let lp ((splits (if split->date
-                       (stable-sort! (xaccAccountGetSplitList acc) less?)
-                       (xaccAccountGetSplitList acc)))
-           (dates (sort dates <))
-           (result '())
-           (last-result nosplit->elt))
-    (match dates
-
-      ;; end of dates. job done!
-      (() (reverse result))
-
-      ((date . rest)
-       (define (before-date? s) (<= (to-date s) date))
-       (define (after-date? s) (< date (to-date s)))
-       (cond
-
-        ;; end of splits, but still has dates. pad with last-result
-        ;; until end of dates.
-        ((null? splits) (lp '() rest (cons last-result result) last-result))
-
-        ;; the next split is still before date.
-        ((and (pair? (cdr splits)) (before-date? (cadr splits)))
-         (lp (cdr splits) dates result (split->elt (car splits))))
-
-        ;; head split after date, accumulate previous result
-        ((after-date? (car splits))
-         (lp splits rest (cons last-result result) last-result))
-
-        ;; head split before date, next split after date, or end.
-        (else
-         (let ((head-result (split->elt (car splits))))
-           (lp (cdr splits) rest (cons head-result result) head-result))))))))
+  (if split->date
+      (gnc-account-accumulate-to-dates acc dates split->elt nosplit->elt split->date)
+      (gnc-account-accumulate-to-dates acc dates split->elt nosplit->elt)))
 
 ;; This works similar as above but returns a commodity-collector,
 ;; thus takes care of children accounts with different currencies.
@@ -669,7 +638,7 @@
 
 ;; function to count the total number of splits to be iterated
 (define (gnc:accounts-count-splits accounts)
-  (fold (lambda (a b) (+ b (length (xaccAccountGetSplitList a)))) 0 accounts))
+  (fold (lambda (a b) (+ b (xaccAccountGetSplitsSize a))) 0 accounts))
 
 ;; Sums up any splits of a certain type affecting a set of accounts.
 ;; the type is an alist '((str "match me") (cased #f) (regexp #f))
@@ -1208,6 +1177,15 @@
           (maybe-str 'Action (xaccSplitGetAction s))
           (maybe-str 'Memo (xaccSplitGetMemo s))))
 
+(define-public (gnc:dump-transaction trans)
+  (format #t "  Transaction:~a Date:~a Currency:~a ~a ~a\n"
+          (string-take (gncTransGetGUID trans) 8)
+          (qof-print-date (xaccTransGetDate trans))
+          (gnc-commodity-get-mnemonic (xaccTransGetCurrency trans))
+          (maybe-str 'Desc (xaccTransGetDescription trans))
+          (maybe-str 'Notes (xaccTransGetNotes trans)))
+  (for-each (cut gnc:dump-split <> #t) (xaccTransGetSplitList trans)))
+
 (define-public (gnc:dump-all-transactions)
   (define query (qof-query-create-for-splits))
   (define (split-has-no-account? split) (null? (xaccSplitGetAccount split)))
@@ -1222,13 +1200,7 @@
       (((? split-has-no-account?) . rest) (lp rest))
       ((split . rest)
        (let ((trans (xaccSplitGetParent split)))
-         (format #t "  Trans ~a: ~a Curr ~a ~a~a\n"
-                 (string-take (gncTransGetGUID trans) 8)
-                 (qof-print-date (xaccTransGetDate trans))
-                 (gnc-commodity-get-mnemonic (xaccTransGetCurrency trans))
-                 (maybe-str 'Desc (xaccTransGetDescription trans))
-                 (maybe-str 'Notes (xaccTransGetNotes trans)))
-         (for-each (cut gnc:dump-split <> #t) (xaccTransGetSplitList trans))
+         (gnc:dump-transaction trans)
          (lp rest))))))
 
 ;; utility function for testing. dumps the whole book contents to
@@ -1242,7 +1214,7 @@
              (gnc-account-get-full-name acc)
              (gnc-commodity-get-mnemonic (xaccAccountGetCommodity acc))
              (xaccAccountGetTypeStr (xaccAccountGetType acc)))
-     (for-each (cut gnc:dump-split <> #f) (xaccAccountGetSplitList acc))
+     (for-each (cut gnc:dump-split <> #f) (xaccAccountGetSplits acc))
      (format #t "         Balance: ~a Cleared: ~a Reconciled: ~a\n"
              (gnc:monetary->string
               (gnc:make-gnc-monetary
@@ -1267,7 +1239,7 @@
                             (gnc-get-current-root-account))))
          (inv-txns (filter (lambda (t) (eqv? (xaccTransGetTxnType t) TXN-TYPE-INVOICE))
                            (map xaccSplitGetParent
-                                (append-map xaccAccountGetSplitList acc-APAR))))
+                                (append-map xaccAccountGetSplits acc-APAR))))
          (invoices (map gncInvoiceGetInvoiceFromTxn inv-txns)))
     (define (maybe-date time64)         ;handle INT-MAX differently
       (if (= 9223372036854775807 time64) "?" (qof-print-date time64)))
